﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

using Unity.MLAgents;
using Unity.MLAgents.Sensors;

public class MLPaddle : Agent
{

    public float _speed = 1;
    public GameObject _ball; 
    private Rigidbody2D _rb;
    private Rigidbody2D _rbball;

    public GameObject _topWall, _downWall;

    ////////////////////////////////
    ////////ML AGENTS METHODS///////
    ////////////////////////////////

    /// <summary>
    /// 
    /// </summary>
    public override void Initialize()
    {
        Debug.Log("Initialize");
        _rb = this.GetComponent<Rigidbody2D>();
        //just in case we forgot to add the ball in the editor
        if (_ball == null) _ball = GameObject.FindGameObjectWithTag("Ball");
        _rbball = _ball.GetComponent<Rigidbody2D>();
    }


    /// <summary>
    /// Just one action and two values: go up or down
    /// </summary>
    /// <param name="vectorAction"></param>
    public override void OnActionReceived(float[] vectorAction)
    {
        Debug.Log("OnActionReceived");
        float lUp = vectorAction[0];

        lUp -= 1; /*Extract 1 from lUp
                   * [-1] down
                   * [0] stay
                   * [1] up */
       


        if ((transform.position.y < _downWall.transform.position.y +
            transform.localScale.y / 2 && lUp < 0))
        {
            /* if the paddle is touching the bottom 
             * border and insisnt in the same direction give 
             * a little penalty. */
            AddReward(-0.05f);
            return;
        }
        if ((lUp > 0) && (transform.position.y > _topWall.transform.position.y -
            transform.localScale.y / 2))
        {
            /* if the paddle is touching the upper 
            * border and insisnt in the same direction give 
            * a little penalty. */
            AddReward(-0.05f);
            return;
        }

        if (lUp != 0)  
        {
            /*If the paddle decides to move, move it, and add 
             * a very very little penalty*/
            _rb.MovePosition(transform.position +
            transform.up * lUp * _speed * Time.deltaTime);
            AddReward(-0.0005f);

        }

    }

    /// <summary>
    /// Collect the observations.
    /// -Vector2d: ball.position. 
    /// -Vector2d: paddle.position. 
    /// </summary>
    /// <param name="sensor"></param>
    public override void CollectObservations(VectorSensor sensor)
    {
        Debug.Log("ColectObservations");
       

        sensor.AddObservation(_ball.transform.localPosition.x);
        sensor.AddObservation(_ball.transform.localPosition.y);
        sensor.AddObservation(_rbball.velocity.x);
        sensor.AddObservation(_rbball.velocity.y);
        sensor.AddObservation(transform.localPosition.y);

        //Total observations 5 positions. 
    }

    /// <summary>
    /// Control the agent with the cursors. 
    /// </summary>
    /// <param name="actionsOut"></param>
    public override void Heuristic(float[] actionsOut)
    {
        
        float lUp = 1f; //Stay

        if (Input.GetKey(KeyCode.UpArrow))
        {
            lUp = 2f; //Up
        }
        if (Input.GetKey(KeyCode.DownArrow))
        {
            lUp = 0f; //Down
        }
        
        /* Put the actions into an array and return
         * remeber that OnActionReceived is going to remove 
         * 1 from lvalue*/
        actionsOut[0] = lUp;

    }

    ////////////////////////////////
    ////////COMMOM METHODS//////////
    ////////////////////////////////
    private void OnTriggerEnter2D(Collider2D collision)
    {
        if (collision.tag == "Ball")
        {
            collision.GetComponent<Ball>().changeXDirection();

            //ADDReward to our agent
            AddReward(2f);
        }

        if (collision.tag == "Wall")
        {

            //ADDpenalty to our agent
            AddReward(-0.05f);
        }
    }

}
